/*!\file:  GetGlobalDofList.cpp
 * \brief create transform matrix for different coordinate systems
 */ 
#include "./elements.h"
#include <math.h>

void CoordinateSystemTransform(double** ptransform,Node** nodes,int numnodes,int dimension){

	int     i;
	double  x_norm,y_norm;
	double *transform         = NULL;
	double *values            = NULL;
	double  coord_system[3][3];

	/*Some checks*/
	_assert_(numnodes && nodes);
	if(dimension!=2 && dimension!=3 && dimension!=4) _error_("only 2d and 3d coordinate systems are supported");

	/*Allocate and initialize transform matrix*/
	transform=(double*)xmalloc(numnodes*dimension*numnodes*dimension*sizeof(double));
	for(i=0;i<numnodes*dimension*numnodes*dimension;i++) transform[i]=0.0;

	/*Create transform matrix for all nodes (x,y for 2d and x,y,z for 3d). It is a block matrix
	 *for 3 nodes:

	 *     | T1 0  0 |
	 * Q = | 0  T2 0 |
	 *     | 0  0  T3|
	 *
	 * Where T1 is the transform matrix for node 1. It is a simple copy of the coordinate system
	 * associated to this node*/
	for(i=0;i<numnodes;i++){
		nodes[i]->GetCoordinateSystem(&coord_system[0][0]);
		if(dimension==2){
			/*We remove the z component, we need to renormalize x and y*/
			x_norm = sqrt( coord_system[0][0]*coord_system[0][0] + coord_system[1][0]*coord_system[1][0]);
			y_norm = sqrt( coord_system[0][1]*coord_system[0][1] + coord_system[1][1]*coord_system[1][1]);
			transform[(dimension*numnodes)*(i*dimension+0) + i*dimension+0] = coord_system[0][0]/x_norm;
			transform[(dimension*numnodes)*(i*dimension+0) + i*dimension+1] = coord_system[0][1]/y_norm;
			transform[(dimension*numnodes)*(i*dimension+1) + i*dimension+0] = coord_system[1][0]/x_norm;
			transform[(dimension*numnodes)*(i*dimension+1) + i*dimension+1] = coord_system[1][1]/y_norm;
		}
		else if(dimension==3){
			transform[(dimension*numnodes)*(i*dimension+0) + i*dimension+0] = coord_system[0][0];
			transform[(dimension*numnodes)*(i*dimension+0) + i*dimension+1] = coord_system[0][1];
			transform[(dimension*numnodes)*(i*dimension+0) + i*dimension+2] = coord_system[0][2];
			transform[(dimension*numnodes)*(i*dimension+1) + i*dimension+0] = coord_system[1][0];
			transform[(dimension*numnodes)*(i*dimension+1) + i*dimension+1] = coord_system[1][1];
			transform[(dimension*numnodes)*(i*dimension+1) + i*dimension+2] = coord_system[1][2];
			transform[(dimension*numnodes)*(i*dimension+2) + i*dimension+0] = coord_system[2][0];
			transform[(dimension*numnodes)*(i*dimension+2) + i*dimension+1] = coord_system[2][1];
			transform[(dimension*numnodes)*(i*dimension+2) + i*dimension+2] = coord_system[2][2];
		}
		else if(dimension==4){
			/*Only the first 3 coordinates are changed (x,y,z), leave the others (P) unchanged*/
			transform[(dimension*numnodes)*(i*dimension+0) + i*dimension+0] = coord_system[0][0];
			transform[(dimension*numnodes)*(i*dimension+0) + i*dimension+1] = coord_system[0][1];
			transform[(dimension*numnodes)*(i*dimension+0) + i*dimension+2] = coord_system[0][2];
			transform[(dimension*numnodes)*(i*dimension+1) + i*dimension+0] = coord_system[1][0];
			transform[(dimension*numnodes)*(i*dimension+1) + i*dimension+1] = coord_system[1][1];
			transform[(dimension*numnodes)*(i*dimension+1) + i*dimension+2] = coord_system[1][2];
			transform[(dimension*numnodes)*(i*dimension+2) + i*dimension+0] = coord_system[2][0];
			transform[(dimension*numnodes)*(i*dimension+2) + i*dimension+1] = coord_system[2][1];
			transform[(dimension*numnodes)*(i*dimension+2) + i*dimension+2] = coord_system[2][2];
			transform[(dimension*numnodes)*(i*dimension+3) + i*dimension+3] = 1.0;
		}
	}

	/*Assign output pointer*/
	*ptransform=transform;
}
